﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Imaging;
using System.IO;

namespace howto_color_matrix
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The original and transformed images.
        private Image OriginalImage = null;
        private Image TransformedImage = null;

        // Disable the items in the Picture menu. (Even though
        // that menu is already disabled, the items' shortcut
        // keys still work if they are enabled.)
        private void Form1_Load(object sender, EventArgs e)
        {
            EnableMenu(mnuPicture, false);
        }

        private Image ApplyColorMatrix(Image image, ColorMatrix color_matrix)
        {
            // Make the result bitmap.
            Bitmap bm = new Bitmap(image.Width, image.Height);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                // Create an ImageAttributes object and use it to
                // draw the original image onto the result image.
                ImageAttributes attr = new ImageAttributes();
                attr.SetColorMatrix(color_matrix);

                Point[] dest_points =
                {
                    new Point(0, 0),
                    new Point(image.Width, 0),
                    new Point(0, image.Height),
                };
                Rectangle source_rect = new Rectangle(
                    0, 0, image.Width, image.Height);
                gr.DrawImage(image, dest_points,
                    source_rect, GraphicsUnit.Pixel, attr);
            }
            return bm;
        }

        private void mnuPictureReset_Click(object sender, EventArgs e)
        {
            TransformedImage = OriginalImage;
            ShowImage();
        }

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdOriginal.ShowDialog() == DialogResult.OK)
            {
                OriginalImage =
                    LoadBitmapUnlocked(ofdOriginal.FileName);
                TransformedImage = OriginalImage;
                ShowImage();
                picImage.Visible = true;

                ClientSize = new Size(
                    picImage.Right + picImage.Left,
                    picImage.Bottom + picImage.Left);

                // Ensable the Picture menu and the Save As command.
                mnuPicture.Enabled = true;
                mnuFileSaveAs.Enabled = true;
                EnableMenu(mnuPicture, true);
            }
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdResult.ShowDialog() == DialogResult.OK)
            {
                SaveImage(TransformedImage, sfdResult.FileName);
            }
        }

        // Multiply red, green, and blue color components by -1.
        private void mnuPictureInvert_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {-1,  0,  0,  0,  0},
                    new float[] { 0, -1,  0,  0,  0},
                    new float[] { 0,  0, -1,  0,  0},
                    new float[] { 0,  0,  0,  1,  0},
                    new float[] { 1,  1,  1,  0,  1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Remove the red component.
        private void mnuPictureRemoveRed_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {0, 0, 0, 0, 0},
                    new float[] {0, 1, 0, 0, 0},
                    new float[] {0, 0, 1, 0, 0},
                    new float[] {0, 0, 0, 1, 0},
                    new float[] {0, 0, 0, 0, 1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Remove the green component.
        private void mnuPictureRemoveGreen_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {1, 0, 0, 0, 0},
                    new float[] {0, 0, 0, 0, 0},
                    new float[] {0, 0, 1, 0, 0},
                    new float[] {0, 0, 0, 1, 0},
                    new float[] {0, 0, 0, 0, 1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Remove the blue component.
        private void mnuPictureRemoveBlue_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {1, 0, 0, 0, 0},
                    new float[] {0, 1, 0, 0, 0},
                    new float[] {0, 0, 0, 0, 0},
                    new float[] {0, 0, 0, 1, 0},
                    new float[] {0, 0, 0, 0, 1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        private void mnuPictureDecreaseAlpha_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {1, 0, 0,    0, 0},
                    new float[] {0, 1, 0,    0, 0},
                    new float[] {0, 0, 1,    0, 0},
                    new float[] {0, 0, 0, 0.8f, 0},
                    new float[] {0, 0, 0,    0, 1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        private void mnuPictureIncreaseAlpha_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {1, 0, 0,     0, 0},
                    new float[] {0, 1, 0,     0, 0},
                    new float[] {0, 0, 1,     0, 0},
                    new float[] {0, 0, 0, 1.25f, 0},
                    new float[] {0, 0, 0,     0, 1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Convert the image to grayscale using weighted component values.
        private void mnuPictureGrayscale_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {0.299f, 0.299f, 0.299f, 0, 0},
                    new float[] {0.587f, 0.587f, 0.587f, 0, 0},
                    new float[] {0.114f, 0.114f, 0.114f, 0, 0},
                    new float[] {0, 0, 0, 1, 0},
                    new float[] {0, 0, 0, 0, 1},
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Convert the image to grayscale by averaging the component values.
        private void mnuPictureAverage_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {0.333f, 0.333f, 0.333f, 0, 0},
                    new float[] {0.333f, 0.333f, 0.333f, 0, 0},
                    new float[] {0.333f, 0.333f, 0.333f, 0, 0},
                    new float[] {0, 0, 0, 1, 0},
                    new float[] {0, 0, 0, 0, 1},
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Increase the color components.
        private void mnuPictureBrighten_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {1.2f,    0,    0, 0, 0},
                    new float[] {   0, 1.2f,    0, 0, 0},
                    new float[] {   0,    0, 1.2f, 0, 0},
                    new float[] {   0,    0,    0, 1, 0},
                    new float[] {   0,    0,    0, 0, 1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Decrease the color components.
        private void mnuPictureDarken_Click(object sender, EventArgs e)
        {
            ColorMatrix color_matrix = new ColorMatrix(
                new float[][]
                {
                    new float[] {0.8f,    0,    0, 0, 0},
                    new float[] {   0, 0.8f,    0, 0, 0},
                    new float[] {   0,    0, 0.8f, 0, 0},
                    new float[] {   0,    0,    0, 1, 0},
                    new float[] {   0,    0,    0, 0, 1}
                });
            TransformedImage = ApplyColorMatrix(TransformedImage, color_matrix);
            ShowImage();
        }

        // Display the transformed image drawn on top of a checkerboard.
        private void ShowImage()
        {
            int wid = TransformedImage.Width;
            int hgt = TransformedImage.Height;
            Bitmap bm = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                // Draw the checkerboard.
                const int step = 20;
                for (int x = 0; x < wid; x += step)
                {
                    for (int y = 0; y < hgt; y += step)
                    {
                        if (((x / step) + (y / step)) % 2 == 0)
                            gr.FillRectangle(Brushes.Red,
                                x, y, step, step);
                        else
                            gr.FillRectangle(Brushes.Yellow,
                                x, y, step, step);
                    }
                }

                // Draw the image on top.
                gr.DrawImage(TransformedImage, new Point(0, 0));
            }

            picImage.Image = bm;
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Enable or disable the items in a menu.
        private void EnableMenu(ToolStripMenuItem mnu, bool enable)
        {
            foreach (ToolStripItem item in mnu.DropDownItems)
                item.Enabled = enable;
        }
    }
}
